import io
import json
import typing as t
from pathlib import Path

import httpx
import typer
from rich.console import Console

try:
    from importlib.metadata import version
except ImportError:
    from importlib_metadata import version

__version__ = version("oxo")

app = typer.Typer()

BASE_URL = typer.Option("https://0x0.st", envvar="OXO_BASE_URL")
TOKEN_CACHE_DIR: t.Optional[str] = typer.Option(None, envvar="OXO_TOKEN_CACHE_DIR")

err_console = Console(stderr=True, color_system=None)


def post_to(
    base_url: str,
    *,
    data: t.Iterable[t.Dict],
    expires: t.Optional[int],
    token_dir: t.Optional[Path],
) -> str:
    retval = []
    with httpx.Client() as client:
        for d in data:
            if isinstance(d.get("file"), io.BufferedReader):
                if expires:
                    res = client.post(base_url, files=d, data={"expires": expires})
                else:
                    res = client.post(base_url, files=d)
                token = res.headers.get("x-token")
                if token:
                    curl_rm = f"curl -Ftoken={token} -Fdelete= {res.text.strip()}"
                    curl_expiry = (
                        f"curl -Ftoken={token} -Fexpires=NUMHOURS {res.text.strip()}"
                    )
                    err_console.print(f"To remove post, {curl_rm}")
                    err_console.print(f"To update expiration date, {curl_rm}")
                    if token_dir:
                        print(f"{d.get('file').name=}")
                        fname = Path(d.get("file").name).name
                        token_dir.joinpath(f"{fname}.token").write_text(
                            json.dumps(
                                dict(
                                    token=token,
                                    curl_rm=curl_rm,
                                    curl_expiry=curl_expiry,
                                )
                            )
                        )
            else:
                res = client.post(base_url, data=d)
            res.raise_for_status()
            retval.append(res.text.strip())
    return " ".join(retval)


def post_files(
    base_url,
    files: t.List[Path],
    *,
    expires: t.Optional[int],
    token_dir: t.Optional[Path],
):
    return post_to(
        base_url,
        data=({"file": f.open("rb")} for f in files),
        expires=expires,
        token_dir=token_dir,
    )


def post_repost(base_url, urls: t.List[str]):
    return post_to(base_url, data=({"url": u.strip()} for u in urls))


def post_shorten(base_url, urls: t.List[str]):
    return post_to(base_url, data=({"shorten": u.strip()} for u in urls))


def version_callback(value: bool):
    if value:
        typer.echo(f"{__version__}")
        raise typer.Exit()


@app.callback()
def main(
    version: t.Optional[bool] = typer.Option(
        None,
        "--version",
        callback=version_callback,
        is_eager=True,
        help="Show the version and exit.",
    ),
):
    """A command line utility for 0x0.st compliant pastebins.

    To use a different 0x0 site, set `OXO_BASE_URL` in your environment,
    or specify it in the relevant subcommand.
    """


@app.command()
def files(
    files: t.List[Path] = typer.Argument(
        ..., min=1, exists=True, file_okay=True, dir_okay=False, resolve_path=True
    ),
    expires: t.Optional[int] = typer.Option(
        None, help="Expiration time, in hours or epoch milliseconds"
    ),
    base_url: str = BASE_URL,
    token_cache_dir: str = TOKEN_CACHE_DIR,
):
    """Upload one or more files.  If one is provided by the 0x0 site used on a
    successful upload, a management token will be printed to stderr along with
    instructions on how to use it to delete or adjust the expiration time on
    the uploaded file(s).  If `--token-cache-dir` is passed or
    `OXO_TOKEN_CACHE_DIR` is set, the management token will also be cached as a
    json file in that directory.  The cache directory will be created on first
    use.

    """
    if token_cache_dir:
        token_dir = Path(token_cache_dir).expanduser().resolve()
        if not token_dir.is_dir():
            typer.secho(
                f"{token_dir=} is not a directory; cannot store tokens here.",
                fg=typer.colors.RED,
                err=True,
            )
            raise typer.Exit(-1)
        token_dir.mkdir(exist_ok=True, parents=True)
    else:
        token_dir = None
    typer.echo(post_files(base_url, files, expires=expires, token_dir=token_dir))


@app.command()
def repost(urls: t.List[str] = typer.Argument(..., min=1), base_url=BASE_URL):
    """Repost one or more urls."""
    typer.echo(post_repost(base_url, urls))


@app.command()
def shorten(urls: t.List[str] = typer.Argument(..., min=1), base_url=BASE_URL):
    """Shorten one or more urls."""
    if base_url == BASE_URL.default:
        typer.secho(
            f"Warning: shortening is often disabled "
            f"for {base_url}, command may fail",
            fg=typer.colors.RED,
            err=True,
        )
    typer.echo(post_shorten(base_url, urls))
